import type { Rule } from '@commitlint/types';
/**
 * Configuration for a single external rule to be enforced if the commit
 * is not from an ignored author.
 */
export interface ExternalRuleToEnforce {
    /**
     * The name of the NPM package that exports the rule.
     * Examples: '@commitlint/rules', 'commitlint-plugin-jira'
     */
    packageName: string;
    /**
     * The name of the rule as exported by the `rules` object of the plugin
     * specified in `packageName`.
     * Example: 'type-enum', 'header-max-length', 'jira-task-id-empty'
     */
    ruleName: string;
    /**
     * The value/options to pass to the target rule. The structure of this
     * value depends entirely on the requirements of the target rule from
     * the specified package.
     */
    value: unknown;
}
/**
 * Options for the `ignoreForAuthors` rule.
 * This rule allows for bypassing a set of specified linting rules if the
 * commit matches an author pattern, or enforcing those rules otherwise.
 */
export interface IgnoreForAuthorsRuleOptions {
    /**
     * An array of string patterns. If any of these patterns are found
     * within the raw commit message (`parsed.raw`), the `rulesToEnforce`
     * (if any) will be skipped, and this main rule will pass.
     * If omitted or empty, author pattern checking is skipped and
     * `rulesToEnforce` are always processed.
     */
    signOffPatternsToIgnore?: string[];
    /**
     * An array of external rule configurations to apply if the commit does
     * not have a `Signed-off-by:` line matching `signOffPatternsToIgnore`.
     * If this array is empty or not provided, and no ignored sign-off is
     * found, this main rule will pass.
     */
    rulesToEnforce?: ExternalRuleToEnforce[];
}
/**
 * A `commitlint` rule named `ignoreForAuthors`.
 * (JSDoc from previous version describing behavior is still applicable)
 * This rule is asynchronous due to its use of dynamic `import()`.
 *
 * @param parsedCommitData - The parsed commit object.
 * @param when - Rule applicability.
 * @param value - Options for this rule.
 * @returns A `Promise<RuleOutcome>`.
 */
export declare const ignoreForAuthors: Rule<IgnoreForAuthorsRuleOptions>;
