/**
 * A `commitlint` rule named `ignoreForAuthors`.
 * (JSDoc from previous version describing behavior is still applicable)
 * This rule is asynchronous due to its use of dynamic `import()`.
 *
 * @param parsedCommitData - The parsed commit object.
 * @param when - Rule applicability.
 * @param value - Options for this rule.
 * @returns A `Promise<RuleOutcome>`.
 */
export const ignoreForAuthors = async (parsedCommitData, when = 'always', value) => {
    if (when === 'never') {
        return [true, ''];
    }
    const parsed = parsedCommitData;
    const options = value || {};
    const { signOffPatternsToIgnore, rulesToEnforce } = options;
    const rawCommitMessage = parsed.raw;
    // noinspection SuspiciousTypeOfGuard
    if (typeof rawCommitMessage !== 'string') {
        return [
            false,
            `Raw commit message is not available or is invalid.`, // Prefix removed
        ];
    }
    if (signOffPatternsToIgnore && Array.isArray(signOffPatternsToIgnore)) {
        // noinspection SuspiciousTypeOfGuard
        if (!signOffPatternsToIgnore.every((pattern) => typeof pattern === 'string')) {
            return [
                false,
                `Configuration error: 'signOffPatternsToIgnore' must be an array of strings.`, // Prefix removed
            ];
        }
        if (signOffPatternsToIgnore.length > 0) {
            const signOffRegex = /^Signed-off-by:\s*(.*)$/gim;
            let match;
            const extractedSignOffs = [];
            while ((match = signOffRegex.exec(rawCommitMessage)) !== null) {
                extractedSignOffs.push(match[1].trim());
            }
            if (extractedSignOffs.length > 0) {
                const matchedPattern = signOffPatternsToIgnore.find((ignorePattern) => extractedSignOffs.some((actualSignOff) => actualSignOff.includes(ignorePattern)));
                if (matchedPattern) {
                    return [
                        true,
                        `Commit has a 'Signed-off-by' line matching pattern ('${matchedPattern}'). Ruleset defined in 'rulesToEnforce' bypassed.`, // Prefix removed
                    ];
                }
            }
        }
    }
    if (!rulesToEnforce ||
        !Array.isArray(rulesToEnforce) ||
        rulesToEnforce.length === 0) {
        return [
            true,
            'No rules configured in "rulesToEnforce" for this author/commit, or author was ignored. Rule passes.', // Prefix removed
        ];
    }
    for (const ruleConfig of rulesToEnforce) {
        // noinspection SuspiciousTypeOfGuard
        if (!ruleConfig ||
            typeof ruleConfig.packageName !== 'string' ||
            typeof ruleConfig.ruleName !== 'string') {
            return [
                false,
                `Invalid rule configuration in 'rulesToEnforce': 'packageName' and 'ruleName' must be strings.`, // Prefix removed
            ];
        }
        const { packageName, ruleName, value: ruleValue } = ruleConfig;
        let ruleFunction;
        try {
            // eslint-disable-next-line no-unsanitized/method
            const targetPluginModule = (await import(packageName));
            let rulesCatalog;
            if (targetPluginModule.default) {
                if (typeof targetPluginModule.default.rules === 'object' &&
                    targetPluginModule.default.rules !== null) {
                    rulesCatalog = targetPluginModule.default.rules;
                }
                else if (typeof targetPluginModule.default === 'object') {
                    rulesCatalog = targetPluginModule.default;
                }
            }
            if (!rulesCatalog &&
                typeof targetPluginModule.rules === 'object' &&
                targetPluginModule.rules !== null) {
                rulesCatalog = targetPluginModule.rules;
            }
            if (rulesCatalog) {
                ruleFunction = rulesCatalog[ruleName];
            }
        }
        catch (error) {
            const errorMessage = error instanceof Error ? error.message : String(error);
            return [
                false,
                `Error loading plugin package '${packageName}': ${errorMessage}`, // Prefix removed
            ];
        }
        if (typeof ruleFunction !== 'function') {
            return [
                false,
                `Rule '${ruleName}' not found or not a function in package '${packageName}'.`, // Prefix removed
            ];
        }
        const outcome = await Promise.resolve(ruleFunction(parsed, 'always', ruleValue));
        const [isValid, messageFromSubRule] = outcome;
        if (!isValid) {
            return [
                false,
                `[via ${packageName}/${ruleName}] ${messageFromSubRule}`, // Kept a short prefix for sub-rule context
            ];
        }
    }
    return [
        true,
        'All configured "rulesToEnforce" passed for this author.', // Prefix removed
    ];
};
//# sourceMappingURL=ignore-for-authors.js.map